#ifndef TOOLMAKER_H
#define TOOLMAKER_H

// My report is flag used to control the reporting of errors in all the makers
// Errors can be reported to a message box, a trace file, both or neither
#ifndef MY_REPORT
#define MY_REPORT
#define REPORT_ERROR(...) toolmaker_report_error(__VA_ARGS__)
// Microsoft code for the message boxes called by REPORT_ERROR - comment out if not wanted.
// If not wanted must also change toolmaker_report_error code to not call the boxes
#include <windows.h>			
#endif

// Tracing of function calls for this maker can be eliminated by selecting appropriate definition for TRACE_CALL
#ifndef MY_TRACE
#define MY_TRACE
#ifndef TRACE_CALL	// select one
#define TRACE_CALL toolmaker_function_trace
//#define TRACE_CALL(...) 
#endif
#endif

// STIX defines functions to handle units and other useful utilities
#include <stp_schema.h>
#include <stix.h>

// ARM defines all the Application Objects in the ARM library (long list)
#include "ARM.h"

// File used by all the makers for tracing - must be defined in main program
extern FILE *pfLog;

// function calls are sent to a trace file or eliminated from code by choosing appropriate define for TRACE_FILE
void toolmaker_function_trace(const char * fmt, ...);

// Errors are reported to message box, the function_trace file, both or neither
void toolmaker_report_error(const char * fmt, ...);

// functions to define simple tool requirements (no value ranges)
User_defined_milling_tool	*	new_apt_tool (RoseDesign *des, const char * id, double tool_diameter, double tool_radius, 
						    double horizontal_distance, double vertical_distance, double unused, double angle, double height, StixUnit length_unit, StixUnit angle_unit);
Endmill				*	new_endmill_tool (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    double num_effective_teeth, double cutting_edge_angle, StixUnit length_unit,  StixUnit angle_unit);
Facemill			*	new_facemill_tool (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    double num_effective_teeth, double cutting_edge_angle, StixUnit length_unit, StixUnit angle_unit);
Tapping_cutting_tool		*	new_tapping_tool (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    const char *thread_form_type, double pitch, double size, StixUnit length_unit);
Spotdrill			*	new_spot_drill (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    double point_angle, StixUnit length_unit, StixUnit angle_unit);
Twist_drill			*	new_twist_drill (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    double point_angle, StixUnit length_unit, StixUnit angle_unit);
Rotating_boring_cutting_tool	*	new_boring_tool (RoseDesign *des, const char * id, double tool_diameter, double overall_assembly_length, double functional_length, double max_depth_of_cut, 
						    StixUnit length_unit);

Machining_tool_IF		*	set_tool_usage (Machining_tool_IF *tool, Workpiece *piece);
Machining_tool_IF		*	set_tool_carousel (Machining_tool_IF *tool, const char * carousel);
Machining_tool_IF		*	set_tool_carousel_position (Machining_tool_IF *tool, const char *position);
Machining_tool_IF		*	set_tool_end_placement (Machining_tool_IF *tool, StixMtrx M);
Machining_tool_IF		*	set_tool_gage_line_placement (Machining_tool_IF *tool, StixMtrx M);

// Set material and standard
Milling_machine_cutting_tool_IF	*	set_tool_cutting_material (Milling_machine_cutting_tool_IF *tool, const char *material, const char *material_standard = "ISO", unsigned component_index = 0);

// set material, life and feed speed recommendations for first cutting_component
Milling_machine_cutting_tool_IF	*	set_tool_expected_life (Milling_machine_cutting_tool_IF *tool, double life, StixUnit time_unit, unsigned component_index = 0);
Milling_machine_cutting_tool_IF	*	set_tool_functional_length (Milling_machine_cutting_tool_IF *tool, double length, StixUnit length_unit, unsigned component_index = 0);
Milling_machine_cutting_tool_IF	*	set_tool_recommended_speed (Milling_machine_cutting_tool_IF *tool, double spindle_speed, StixUnit speed_unit, unsigned component_index = 0);
Milling_machine_cutting_tool_IF	*	set_tool_recommended_feed (Milling_machine_cutting_tool_IF *tool, double feedrate, StixUnit feed_unit, unsigned component_index = 0);

// set  parameters with limits
Milling_machine_cutting_tool_IF	*	set_tool_expected_life_lower_and_upper (Milling_machine_cutting_tool_IF *tool, double life, StixUnit time_unit, double lower, const char * lreason, double upper, const char * ureason, unsigned component_index = 0);
Milling_machine_cutting_tool_IF	*	set_tool_functional_length_lower_and_upper (Milling_machine_cutting_tool_IF *tool, double length, StixUnit length_unit, double lower, const char * lreason, double upper, const char * ureason, unsigned component_index = 0);
Milling_machine_cutting_tool_IF	*	set_tool_recommended_speed_lower_and_upper (Milling_machine_cutting_tool_IF *tool, double spindle_speed, StixUnit speed_unit, double lower, const char * lreason, double upper, const char * ureason, unsigned component_index = 0);
Milling_machine_cutting_tool_IF	*	set_tool_recommended_feed_lower_and_upper (Milling_machine_cutting_tool_IF *tool, double feedrate, StixUnit feed_unit, double lower, const char * lreason, double upper, const char * ureason, unsigned component_index = 0);

// set number of cutting components - give each same data as first component
Milling_machine_cutting_tool_IF	*	set_tool_number_of_flutes (Milling_machine_cutting_tool_IF *tool, unsigned flute_count);

// set major tool parameters
Milling_machine_cutting_tool_IF	*	set_tool_overall_length (Milling_machine_cutting_tool_IF *tool, double length, StixUnit length_unit);
Milling_machine_cutting_tool_IF	*	set_tool_flute_length (Milling_machine_cutting_tool_IF *tool, double length, StixUnit length_unit);
Milling_machine_cutting_tool_IF	*	set_tool_diameter (Milling_machine_cutting_tool_IF *tool, double diameter, StixUnit length_unit);
Milling_cutting_tool_IF		*	set_tool_radius (Milling_cutting_tool_IF *tool, double radius, StixUnit length_unit);
Endmill				*	set_tool_taper_angle (Endmill *tool, double angle, StixUnit angle_unit);
Facemill			*	set_tool_taper_angle (Facemill *tool, double angle, StixUnit angle_unit);
Drilling_cutting_tool_IF	*	set_tool_tip_angle (Drilling_cutting_tool_IF *tool, double angle, StixUnit angle_unit);

    // set tool parameters with limits
    RoseBoolean set_tool_overall_assembly_length_lower_and_upper (int ws_id, double length, double lower, const char * lreason, double upper, const char * ureason);
    RoseBoolean set_tool_flute_length_lower_and_upper (int ws_id, double length, double lower, const char * lreason, double upper, const char * ureason);
    RoseBoolean set_tool_diameter_lower_and_upper (int ws_id, double diameter, double lower, const char * lreason, double upper, const char * ureason);
    RoseBoolean set_tool_radius_lower_and_upper  (int ws_id, double radius, double lower, const char * lreason, double upper, const char * ureason);
    RoseBoolean set_tool_taper_angle_lower_and_upper  (int ws_id, double angle, double lower, const char * lreason, double upper, const char * ureason);
    RoseBoolean set_tool_tip_angle_lower_and_upper  (int ws_id, double angle, double lower, const char * lreason, double upper, const char * ureason);

Milling_machine_cutting_tool_IF	*	set_tool_hand_of_cut (Milling_machine_cutting_tool_IF *tool,  const char* left_neutral_right);
Milling_machine_cutting_tool_IF	*	set_tool_coolant_through_tool (Milling_machine_cutting_tool_IF *tool, int on_off);

// Tapping tool parameters
Tapping_cutting_tool		*	set_tool_thread_form_type (Tapping_cutting_tool *tool, const char *form_type);
Tapping_cutting_tool		*	set_tool_thread_size (Tapping_cutting_tool *tool, double size, StixUnit length_unit);
Tapping_cutting_tool		*	set_tool_thread_pitch (Tapping_cutting_tool *tool, double pitch, StixUnit length_unit);
Tapping_cutting_tool		*	set_tool_thread_taper_count (Tapping_cutting_tool *tool, double count);

    // Tool parameters with limits
    RoseBoolean set_tool_thread_size_lower_and_upper (int ws_id, double size, double lower, const char * lreason, double upper, const char * ureason);
    RoseBoolean set_tool_thread_pitch_lower_and_upper (int ws_id, double pitch, double lower, const char * lreason, double upper, const char * ureason);
    RoseBoolean set_tool_taper_thread_count_lower_and_upper (int ws_id, double count, double lower, const char * lreason, double upper, const char * ureason);


#endif